@@ -52,6 +52,7 @@
 
 #if defined(USE_EIGEN)
 #include "matrix_cholesky.hpp"
+#include "least_squares.hpp"
 #endif
 
 #include "matrix_invert.hpp"
@@ -79,13 +80,14 @@ void LibInit_jmg()
 				   KLISTEND};
   new DLibFun(lib::routine_names_value,string("ROUTINE_NAMES"),-1,routine_namesKey);
   
-  // the 2 following by Alain C. and Levan L.
-  const string isaKey[]={"ARRAY","FILE","NULL","NUMBER","SCALAR",KLISTEND};
+  // the 2 following by Alain C. and Levan L., Summer 2014
+  const string isaKey[]={"ARRAY","FILE","NULL","NUMBER","SCALAR","BOOLEAN",
+			 "COMPLEX","FLOAT","INTEGER","STRING",KLISTEND};
   new DLibFunRetNew(lib::isa_fun,string("ISA"),2,isaKey);
 
   new DLibFunRetNew(lib::typename_fun,string("TYPENAME"),1);
 
-  // the following (Chol and Invert) by Alain C. and Nodar K.
+  // the following (Chol and Invert) by Alain C. and Nodar K., Summer 2013
   // Since we have a difference between the IDL way and the Eigen way
   // we (temporary) remove these 2 codes in the CVS of GDL
   // Help welcome. 
@@ -99,6 +101,11 @@ void LibInit_jmg()
   new DLibPro(lib::la_choldc_pro,string("LA_CHOLDC"),1,lacholdcKey);
   const string lacholsolKey[]={"DOUBLE","STATUS",KLISTEND};
   new DLibFunRetNew(lib::la_cholsol_fun,string("LA_CHOLSOL"),2,lacholsolKey);
+
+  // Ilia N. & Alain C., Summer 2015
+  const string laleastsquaresKey[]={"DOUBLE","METHOD","RANK","RCONDITION","RESIDUAL","STATUS",KLISTEND};
+  new DLibFunRetNew(lib::la_least_squares_fun,string("LA_LEAST_SQUARES"),2,laleastsquaresKey);
+
 #endif
 
 #if defined(HAVE_LIBGSL) && defined(HAVE_LIBGSLCBLAS)
@@ -116,8 +123,8 @@ void LibInit_jmg()
 
   const string randomKey[]={"DOUBLE","GAMMA","LONG","NORMAL",
 			    "BINOMIAL","POISSON","UNIFORM",KLISTEND};
-  new DLibFunRetNew(lib::random_fun,string("RANDOMU"),MAXRANK,randomKey);
-  new DLibFunRetNew(lib::random_fun,string("RANDOMN"),MAXRANK,randomKey);
+  new DLibFunRetNew(lib::random_fun,string("RANDOMU"),MAXRANK+1,randomKey);
+  new DLibFunRetNew(lib::random_fun,string("RANDOMN"),MAXRANK+1,randomKey);
 
   const string checkmathKey[]={"MASK","NOCLEAR","PRINT",KLISTEND};
   new DLibFunRetNew(lib::check_math_fun,string("CHECK_MATH"),2,checkmathKey);
@@ -125,7 +132,8 @@ void LibInit_jmg()
   const string histogramKey[]={"BINSIZE","INPUT","MAX","MIN","NBINS",
 			       "OMAX","OMIN","REVERSE_INDICES",
 			       "LOCATIONS","NAN",KLISTEND};
-  new DLibFunRetNew(lib::histogram_fun,string("HISTOGRAM"),1,histogramKey);
+  const string histogramWarnKey[]={"L64",KLISTEND};
+  new DLibFunRetNew(lib::histogram_fun,string("HISTOGRAM"),1,histogramKey,histogramWarnKey);
 
   const string interpolateKey[]={"CUBIC","GRID","MISSING","NEAREST_NEIGHBOUR",KLISTEND};
   new DLibFunRetNew(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey);
@@ -210,15 +218,16 @@ void LibInit_jmg()
   new DLibFunRetNew(lib::make_array,string("MAKE_ARRAY"),MAXRANK,make_arrayKey);
 
   const string reformKey[]={"OVERWRITE",KLISTEND};
-  new DLibFun(lib::reform,string("REFORM"),MAXRANK,reformKey);
+  new DLibFun(lib::reform,string("REFORM"),MAXRANK+1,reformKey);
 
   new DLibPro(lib::point_lun,string("POINT_LUN"),2);
 
   new DLibPro(lib::linkimage,string("LINKIMAGE"),4);
 
-  new DLibPro(lib::wait,string("WAIT"),1);
+  new DLibPro(lib::wait_pro,string("WAIT"),1);
 
 #if defined(USE_HDF)
+  new DLibFunRetNew(lib::hdf_ishdf,string("HDF_ISHDF"),1);
   const string hdf_openKey[]={"READ","RDWR","CREATE","ALL","NUM_DD","WRITE",KLISTEND};
   new DLibFunRetNew(lib::hdf_open_fun,string("HDF_OPEN"),2,hdf_openKey);
 
@@ -322,148 +331,208 @@ void LibInit_jmg()
   new DLibFunRetNew(lib::call_external, string("CALL_EXTERNAL"), -1, call_externalKey);
 
 // WIDGET_ functions
+  new DLibPro(lib::widget_displaycontextmenu,string("WIDGET_DISPLAYCONTEXTMENU"),4);
+  
 #define WIDGET_COMMON_KEYWORDS "EVENT_FUNC", "EVENT_PRO", "FUNC_GET_VALUE", "KILL_NOTIFY" \
     , "NOTIFY_REALIZE" , "PRO_SET_VALUE" , "NO_COPY" , "GROUP_LEADER"\
     , "SCR_XSIZE" , "SCR_YSIZE" , "SCROLL" , "SENSITIVE" \
     , "UNAME" , "UNITS" , "UVALUE" , "XOFFSET" \
-    , "XSIZE" , "YOFFSET" , "YSIZE" , "FRAME"
+    , "XSIZE" , "YOFFSET" , "YSIZE" , "FRAME" \
+    , "ALIGN_LEFT", "ALIGN_RIGHT", "ALIGN_CENTER", "ALIGN_BOTTOM", "ALIGN_TOP", "FONT","RESOURCE_NAME"
 
-  const string widget_drawKey[] = { WIDGET_COMMON_KEYWORDS
-    , "APP_SCROLL"
-    , "BUTTON_EVENTS"
-    , "CLASSNAME"
+//ACTIVEX
+//BASE  
+  const string widget_baseKey[] = {WIDGET_COMMON_KEYWORDS,"MBAR","APP_MBAR","MODAL","COLUMN","ROW",
+  "EXCLUSIVE","NONEXCLUSIVE","FLOATING","MAP","TITLE","XPAD","X_SCROLL_SIZE","YPAD","Y_SCROLL_SIZE","DISPLAY_NAME",
+  "RNAME_MBAR","TAB_MODE","CONTEXT_EVENTS","KBRD_FOCUS_EVENTS","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS",
+  "TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TRACKING_EVENTS","GRID_LAYOUT",
+  "BASE_ALIGN_CENTER","BASE_ALIGN_LEFT","BASE_ALIGN_RIGHT","BASE_ALIGN_BOTTOM","BASE_ALIGN_TOP","SPACE","CONTEXT_MENU",KLISTEND};
+  const string widget_baseWarnKey[] = {"TLB_FRAME_ATTR",  "TOOLBAR", "BITMAP", KLISTEND};
+  new DLibFunRetNew(lib::widget_base,string("WIDGET_BASE"),1,widget_baseKey,widget_baseWarnKey);
+//BUTTON
+  const string widget_buttonKey[] = {WIDGET_COMMON_KEYWORDS,"MENU","VALUE","HELP","SEPARATOR","INPUT_FOCUS","BITMAP","TOOLTIP", "TRACKING_EVENTS","DYNAMIC_RESIZE",KLISTEND};
+  const string widget_buttonWarnKey[] ={"ACCELERATOR","CHECKED_MENU",
+ "X_BITMAP_EXTRA","TAB_MODE","NO_RELEASE",KLISTEND};
+  new DLibFunRetNew(lib::widget_button,string("WIDGET_BUTTON"),1,widget_buttonKey,widget_buttonWarnKey);
+//COMBOBOX
+  const string widget_comboboxKey[] = {WIDGET_COMMON_KEYWORDS,"EDITABLE","TITLE","VALUE","TRACKING_EVENTS","DYNAMIC_RESIZE",KLISTEND};
+//  const string widget_comboboxWarnKey[] = {"TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_combobox,string("WIDGET_COMBOBOX"),1,widget_comboboxKey);//,widget_comboboxWarnKey);
+//CONTROL
+  const string widget_ControlKey[] = {"REALIZE","MANAGED","EVENT_FUNC","EVENT_PRO",
+  "XMANAGER_ACTIVE_COMMAND","DESTROY","DELAY_DESTROY",
+  "GET_UVALUE","SET_UVALUE","SET_VALUE",
+  "MAP","FUNC_GET_VALUE","PRO_SET_VALUE",
+  "SET_UNAME","NO_COPY","SET_BUTTON",
+  "SET_DROPLIST_SELECT","SET_LIST_SELECT","SET_COMBOBOX_SELECT",
+  "SENSITIVE","XOFFSET","YOFFSET",
+  "GET_VALUE","NO_NEWLINE","TLB_GET_SIZE",
+  "HOURGLASS","TLB_SET_TITLE","INPUT_FOCUS",
+  "CLEAR_EVENTS","TLB_SET_XOFFSET","TLB_SET_YOFFSET","TLB_GET_OFFSET",
+  "DRAW_BUTTON_EVENTS","DRAW_EXPOSE_EVENTS","DRAW_KEYBOARD_EVENTS", 
+  "DRAW_MOTION_EVENTS","DRAW_WHEEL_EVENTS","TRACKING_EVENTS","DRAW_VIEWPORT_EVENTS",
+  "SET_DROP_EVENTS","KILL_NOTIFY","SHOW","APPEND","USE_TEXT_SELECT","SET_TEXT_SELECT",
+  "XSIZE","YSIZE","SCR_XSIZE","SCR_YSIZE","DRAW_XSIZE","DRAW_YSIZE","FRAME","SCROLL",
+  "BITMAP","TIMER","NOTIFY_REALIZE","ALL_TABLE_EVENTS","ALL_TEXT_EVENTS",
+  "TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TLB_ICONIFY_EVENTS",
+  "CONTEXT_EVENTS","KBRD_FOCUS_EVENTS","ALIGNMENT",
+  "BACKGROUND_COLOR","FOREGROUND_COLOR","ROW_LABELS","COLUMN_LABELS",
+  "COLUMN_WIDTHS","ROW_HEIGHTS","TABLE_DISJOINT_SELECTION","USE_TABLE_SELECT",
+  "INSERT_COLUMNS","INSERT_ROWS","DELETE_COLUMNS","DELETE_ROWS",
+  "AM_PM", "DAYS_OF_WEEK", "MONTHS",  "SET_TABLE_SELECT","SET_TABLE_VIEW",
+  "UPDATE","FORMAT","EDIT_CELL",  "TABLE_XSIZE","TABLE_YSIZE","SEND_EVENT","BAD_ID",
+  "GROUP_LEADER", "COMBOBOX_ADDITEM" ,"COMBOBOX_DELETEITEM" ,"COMBOBOX_INDEX", 
+  "GET_DRAW_VIEW","SET_TAB_CURRENT", "UNITS","DYNAMIC_RESIZE",
+  KLISTEND};
+  const string widget_WarnControlKey[] ={"DEFAULT_FONT",
+  "PUSHBUTTON_EVENTS","TABLE_BLANK","TAB_MODE","SET_TAB_MULTILINE",KLISTEND}; //LIST NOT CLOSE!!!  
+  //IMPORTANT :   
+  new DLibPro(lib::widget_control,string("WIDGET_CONTROL"),1, 
+	      widget_ControlKey,widget_WarnControlKey);
+//DISPLAYCONTEXTMENU
+//DRAW
+  const string widget_drawKey[] = { WIDGET_COMMON_KEYWORDS, "X_SCROLL_SIZE", "Y_SCROLL_SIZE"
+  , "MOTION_EVENTS"
+  , "BUTTON_EVENTS"
+  , "DROP_EVENTS"
+  , "EXPOSE_EVENTS"
+  , "KEYBOARD_EVENTS"
+  , "TRACKING_EVENTS"
+  , "WHEEL_EVENTS"
+  , "VIEWPORT_EVENTS"
+  , "INPUT_FOCUS"
+  , "APP_SCROLL"
+  , "TOOLTIP"
+  , "RETAIN"  //not taken into account, but not useful, too.
+  , KLISTEND};
+   const string widget_drawWarnKey[] = {
+     "CLASSNAME"
     , "COLOR_MODEL"
     , "COLORS"
-    , "DRAG_NOTIFY"
-    , "DROP_EVENTS"
-    , "EXPOSE_EVENTS"
+//    , "DRAG_NOTIFY" //should be implemented 
     , "GRAPHICS_LEVEL"
     , "IGNORE_ACCELERATORS"
-    , "KEYBOARD_EVENTS"
-    , "MOTION_EVENTS"
     , "RENDERER"
-    , "RESOURCE_NAME"
-    , "RETAIN"
-    , "TOOLTIP"
-    , "TRACKING_EVENTS"
-    , "VIEWPORT_EVENTS"
-    , "WHEEL_EVENTS"
-    , "X_SCROLL_SIZE"
-    , "Y_SCROLL_SIZE"
     ,KLISTEND};
-  new DLibFunRetNew(lib::widget_draw,string("WIDGET_DRAW"),1,widget_drawKey);
-  
+  new DLibFunRetNew(lib::widget_draw,string("WIDGET_DRAW"),1,widget_drawKey,widget_drawWarnKey);
+ //DROPLIST 
+  const string widget_droplistKey[] = {WIDGET_COMMON_KEYWORDS,"TITLE","VALUE","TRACKING_EVENTS","DYNAMIC_RESIZE",KLISTEND};
+//  const string widget_droplistWarnKey[] = {"TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_droplist,string("WIDGET_DROPLIST"),1,widget_droplistKey);//,widget_droplistWarnKey);
+//EVENT  
+  const string widget_eventKey[] = {"XMANAGER_BLOCK","DESTROY","SAVE_HOURGLASS","NOWAIT","BAD_ID",KLISTEND};
+  new DLibFunRetNew(lib::widget_event,string("WIDGET_EVENT"),1,widget_eventKey); //complete!!
+//INFO
+  const string widget_infoKey[] = {  "ACTIVE","VALID_ID","MODAL","MANAGED",
+  "XMANAGER_BLOCK", //only GDL, used in xmanager.pro , may even not be useful now.
+  "CHILD","VERSION","GEOMETRY","UNAME",
+  "FONTNAME",//not really supported - returns "".
+  "BUTTON_SET","PARENT","TEXT_SELECT","FIND_BY_UNAME","TYPE","NAME",
+  "TABLE_DISJOINT_SELECTION","TABLE_SELECT","COLUMN_WIDTHS","ROW_HEIGHTS","USE_TABLE_SELECT","SYSTEM_COLORS",
+  "TREE_ROOT","TREE_SELECT","TREE_EXPANDED","TREE_FOLDER","TREE_INDEX","TREE_BITMAP","DROP_EVENTS","DRAGGABLE","DRAG_NOTIFY","MASK",
+  "LIST_SELECT","DROPLIST_SELECT","COMBOBOX_GETTEXT",
+  "TAB_NUMBER","TAB_MULTILINE","TAB_CURRENT","UPDATE",
+  "EVENT_FUNC","EVENT_PRO","N_CHILDREN","ALL_CHILDREN","REALIZED", "UNITS",KLISTEND};
+  const string widget_infoWarnKey[]={
+//  "COLUMN_WIDTHS",
+//  "COMBOBOX_NUMBER","COMPONENT","CONTEXT_EVENTS",
+//  "DRAW_BUTTON_EVENTS","DRAW_EXPOSE_EVENTS","DRAW_KEYBOARD_EVENTS","DRAW_MOTION_EVENTS",
+//  "DRAW_VIEWPORT_EVENTS","DRAW_WHEEL_EVENTS","DROPLIST_NUMBER","DROPLIST_SELECT",
+//  "DYNAMIC_RESIZE","EVENT_FUNC","EVENT_PRO","FIND_BY_UNAME","KBRD_FOCUS_EVENTS",
+//  "LIST_MULTIPLE","LIST_NUMBER","LIST_NUM_VISIBLE","LIST_TOP","MAP",,
+//  "MULTIPLE_PROPERTIES",,"PROPERTYSHEET_NSELECTED","PROPERTYSHEET_SELECTED",
+//  "PROPERTY_VALID","PROPERTY_VALUE","PUSHBUTTON_EVENTS","ROW_HEIGHTS","SENSITIVE","SIBLING",
+//  "SLIDER_MIN_MAX","STRING_SIZE","TABLE_ALL_EVENTS","TABLE_BACKGROUND_COLOR",
+//  "TABLE_EDITABLE","TABLE_EDIT_CELL","TABLE_FONT","TABLE_FOREGROUND_COLOR","TABLE_VIEW",
+//  "TAB_MODE","TEXT_ALL_EVENTS","TEXT_EDITABLE","TEXT_NUMBER","TEXT_OFFSET_TO_XY",
+//  "TEXT_TOP_LINE","TEXT_XY_TO_OFFSET","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS",
+//  "TLB_SIZE_EVENTS","TOOLTIP","TRACKING_EVENTS","TREE_BITMAP","TREE_DRAG_SELECT","UPDATE","USE_TABLE_SELECT","VISIBLE",
+  "TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_info,string("WIDGET_INFO"),1,widget_infoKey,widget_infoWarnKey);
+//LABEL
+  const string widget_labelKey[] = {WIDGET_COMMON_KEYWORDS,"VALUE","SUNKEN_FRAME","TRACKING_EVENTS","DYNAMIC_RESIZE",KLISTEND};
+  new DLibFunRetNew(lib::widget_label,string("WIDGET_LABEL"),1,widget_labelKey);
+//LIST
+  const string widget_listKey[] = {WIDGET_COMMON_KEYWORDS,"MULTIPLE","VALUE","CONTEXT_EVENTS","TRACKING_EVENTS",KLISTEND};
+  const string widget_listWarnKey[] = {"TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_list,string("WIDGET_LIST"),1,widget_listKey,widget_listWarnKey);
+//MESSAGE
+//PROPERTYSHEET (2-columns TABLE)
+#ifdef HAVE_WXWIDGETS_PROPERTYGRID
+  const string widget_propertyKey[] = {WIDGET_COMMON_KEYWORDS,KLISTEND} //to be finished...
+  new DLibFunRetNew(lib::widget_property,string("WIDGET_PROPERTYSHEET"),1,widget_propertyKey);
+#endif
+  //SLIDER
+  const string widget_sliderKey[] = {WIDGET_COMMON_KEYWORDS,"TITLE","DRAG","VALUE","MINIMUM","MAXIMUM","VERTICAL",
+  "SUPPRESS_VALUE", "TRACKING_EVENTS", KLISTEND};
+  const string widget_sliderWarnKey[] = {"TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_slider,string("WIDGET_SLIDER"),1,widget_sliderKey,widget_sliderWarnKey);
+//STUB
+//TAB
+  const string widget_tabKey[] = {WIDGET_COMMON_KEYWORDS,"MULTILINE","LOCATION","TRACKING_EVENTS", KLISTEND};
+//  const string widget_tabWarnKey[] = {WIDGET_COMMON_KEYWORDS,"TAB_MODE", KLISTEND};
+  new DLibFunRetNew(lib::widget_tab,string("WIDGET_TAB"),1,widget_tabKey); //,widget_tabWarnKey);
+//TABLE
   const string widget_tableKey[] = {WIDGET_COMMON_KEYWORDS
   , "ALIGNMENT"
   , "ALL_EVENTS"
-  , "AM_PM"
   , "BACKGROUND_COLOR"
   , "COLUMN_LABELS"
-  , "COLUMN_MAJOR"
-  , "ROW_MAJOR"
   , "COLUMN_WIDTHS"
   , "CONTEXT_EVENTS"
-  , "DAYS_OF_WEEK"
   , "DISJOINT_SELECTION"
   , "EDITABLE"
-  , "FONT"
   , "FOREGROUND_COLOR"
   , "FORMAT"
-  , "GROUP_LEADER"
-  , "IGNORE_ACCELERATORS"
   , "KBRD_FOCUS_EVENTS"
-  , "MONTHS"
-  , "NO_COLUMN_HEADERS"
-  , "NO_HEADERS"
-  , "NO_ROW_HEADERS"
   , "RESIZEABLE_COLUMNS"
   , "RESIZEABLE_ROWS"
   , "ROW_HEIGHTS"
   , "ROW_LABELS"
-  , "TAB_MODE"
   , "TRACKING_EVENTS"
   , "VALUE"
   , "X_SCROLL_SIZE"
   , "Y_SCROLL_SIZE"
+  , "NO_COLUMN_HEADERS"
+  , "NO_HEADERS"
+  , "NO_ROW_HEADERS"
+  , "COLUMN_MAJOR"
+  , "ROW_MAJOR"
+  , "AM_PM"
+  , "DAYS_OF_WEEK"
+  , "MONTHS"
   , KLISTEND};
-  new DLibFunRetNew(lib::widget_table,string("WIDGET_TABLE"),1,widget_tableKey);
-  
-  const string widget_treeKey[] = {WIDGET_COMMON_KEYWORDS
-  , "ALIGN_BOTTOM"
-  , "ALIGN_CENTER"
-  , "ALIGN_LEFT"
-  , "ALIGN_RIGHT"
-  , "ALIGN_TOP"
-  , "BITMAP"
-  , "CHECKBOX"
-  , "CHECKED"
-  , "DRAG_NOTIFY"
-  , "DRAGGABLE"
-  , "EXPANDED"
-  , "FOLDER"
-  , "GROUP_LEADER"
-  , "INDEX"
+  const string widget_tableWarnKey[] = {
+  "IGNORE_ACCELERATORS"
+  , "TAB_MODE"
+  , KLISTEND};
+  new DLibFunRetNew(lib::widget_table,string("WIDGET_TABLE"),1,widget_tableKey,widget_tableWarnKey);
+//TEXT
+  const string widget_textKey[] = {WIDGET_COMMON_KEYWORDS,"EDITABLE","NO_NEWLINE","VALUE",
+  "INPUT_FOCUS","ALL_EVENTS","CONTEXT_EVENTS","KBRD_FOCUS_EVENTS","TRACKING_EVENTS",KLISTEND};
+  const string widget_textWarnKey[] = {"IGNORE_ACCELERATORS","TAB_MODE","WRAP",KLISTEND};
+  new DLibFunRetNew(lib::widget_text,string("WIDGET_TEXT"),1,widget_textKey,widget_textWarnKey);
+//TREE
+  const string widget_treeWarnKey[] = { "DRAG_NOTIFY"
   , "MASK"
   , "MULTIPLE"
-  , "NO_BITMAPS"
   , "TAB_MODE"
-  , "TOOLTIP"
   , KLISTEND};
-//   , "CONTEXT_EVENTS"
-//   , "TRACKING_EVENTS"
-//   , "DROP_EVENTS"
-  new DLibFunRetNew(lib::widget_base,string("WIDGET_TREE"),1,widget_treeKey);
-  
-  const string widget_baseKey[] = {WIDGET_COMMON_KEYWORDS,"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MBAR","MODAL","BASE_ALIGN_BOTTOM","BASE_ALIGN_CENTER","BASE_ALIGN_LEFT","BASE_ALIGN_RIGHT","BASE_ALIGN_TOP","COLUMN","ROW","CONTEXT_EVENTS","CONTEXT_MENU","EXCLUSIVE","NONEXCLUSIVE","FLOATING","GRID_LAYOUT","KBRD_FOCUS_EVENTS","MAP","SPACE","TITLE","TLB_FRAME_ATTR","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TOOLBAR","TRACKING_EVENTS","XPAD","X_SCROLL_SIZE","YPAD","Y_SCROLL_SIZE","DISPLAY_NAME","RESOURCE_NAME","RNAME_MBAR","TAB_MODE",KLISTEND};
-  new DLibFunRetNew(lib::widget_base,string("WIDGET_BASE"),1,widget_baseKey);
-
-  const string widget_buttonKey[] = {WIDGET_COMMON_KEYWORDS,"ACCELERATOR","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","BITMAP","CHECKED_MENU","DYNAMIC_RESIZE","FONT","HELP","MENU","NO_COPY","NO_RELEASE","PUSHBUTTON_EVENTS","SEPARATOR","TAB_MODE","TOOLTIP","TRACKING_EVENTS","VALUE","X_BITMAP_EXTRA",KLISTEND};
-  new DLibFunRetNew(lib::widget_button,string("WIDGET_BUTTON"),1,widget_buttonKey);
-
-  const string widget_droplistKey[] = {WIDGET_COMMON_KEYWORDS,"DYNAMIC_RESIZE","FONT","RESOURCE_NAME","TAB_MODE","TITLE","TRACKING_EVENTS","VALUE",KLISTEND};
-  new DLibFunRetNew(lib::widget_droplist,string("WIDGET_DROPLIST"),1,widget_droplistKey);
-
-  const string widget_comboboxKey[] = {WIDGET_COMMON_KEYWORDS,"EDITABLE","DYNAMIC_RESIZE","FONT","RESOURCE_NAME","TAB_MODE","TITLE","TRACKING_EVENTS","VALUE",KLISTEND};
-  new DLibFunRetNew(lib::widget_combobox,string("WIDGET_COMBOBOX"),1,widget_comboboxKey);
-
-  const string widget_listKey[] = {WIDGET_COMMON_KEYWORDS,"CONTEXT_EVENTS","FONT","MULTIPLE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE",KLISTEND};
-  new DLibFunRetNew(lib::widget_list,string("WIDGET_LIST"),1,widget_listKey);
-
+  const string widget_treeKey[] = {WIDGET_COMMON_KEYWORDS 
+  , "BITMAP"
+  , "FOLDER"
+  , "EXPANDED"
+  , "VALUE"
+  , "CONTEXT_EVENTS"
+  , "TRACKING_EVENTS"
+  , "DROP_EVENTS"
+  , "DRAGGABLE"
+  , "INDEX"
+  , KLISTEND}; 
+  new DLibFunRetNew(lib::widget_tree,string("WIDGET_TREE"),1,widget_treeKey,widget_treeWarnKey);
+//TREE_MOVE  
 // 	const string widget_bgroupKey[] =
-// 	{"BUTTON_UVALUE","COLUMN","EVENT_FUNC","EXCLUSIVE","NONEXCLUSIVE","SPACE","XPAD","YPAD","FONT","FRAME","IDS","LABEL_LEFT","LABEL_TOP","MAP","NO_RELEASE","RETURN_ID","RETURN_INDEX","RETURN_NAME","ROW","SCROLL","SET_VALUE","TAB_MODE","X_SCROLL_SIZE","Y_SCROLL_SIZE","SET_VALUE","UNAME","UVALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
+// 	{"BUTTON_UVALUE","COLUMN","EVENT_FUNC","EXCLUSIVE","NONEXCLUSIVE","SPACE","XPAD","YPAD","FRAME","IDS","LABEL_LEFT","LABEL_TOP","MAP","NO_RELEASE","RETURN_ID","RETURN_INDEX","RETURN_NAME","ROW","SCROLL","SET_VALUE","TAB_MODE","X_SCROLL_SIZE","Y_SCROLL_SIZE","SET_VALUE","UNAME","UVALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
 // 	new DLibFunRetNew(lib::widget_bgroup,string("CW_BGROUP"),2,widget_bgroupKey);
 
-  const string widget_textKey[] = {WIDGET_COMMON_KEYWORDS,"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","FONT","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","NO_NEWLINE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE","WRAP",KLISTEND};
-  new DLibFunRetNew(lib::widget_text,string("WIDGET_TEXT"),1,widget_textKey);
-
-  const string widget_labelKey[] = {WIDGET_COMMON_KEYWORDS,"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","FONT","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","NO_NEWLINE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE","WRAP",KLISTEND};
-  const string widget_labelWarnKey[] = {"ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT", KLISTEND};
-  new DLibFunRetNew(lib::widget_label,string("WIDGET_LABEL"),1,widget_labelKey,widget_labelWarnKey);
-
-  const string widget_tabKey[] = {WIDGET_COMMON_KEYWORDS,"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MULTILINE","LOCATION","TAB_MODE","TRACKING_EVENTS", KLISTEND};
-  new DLibFunRetNew(lib::widget_tab,string("WIDGET_TAB"),1,widget_tabKey);
-
-  const string widget_slider[] = {WIDGET_COMMON_KEYWORDS,"TITLE"/*TODO*/,"DRAG","VALUE","MINIMUM","MAXIMUM","VERTICAL","SUPPRESS_VALUE", KLISTEND};
-  new DLibFunRetNew(lib::widget_slider,string("WIDGET_SLIDER"),1,widget_slider);
-
-  
-  const string widget_infoKey[] = {"VALID_ID","MODAL","MANAGED","XMANAGER_BLOCK","CHILD","VERSION","GEOMETRY","UNAME",
-				   KLISTEND};
-  new DLibFunRetNew(lib::widget_info,string("WIDGET_INFO"),1,widget_infoKey);
-
-  const string widget_eventKey[] = {"XMANAGER_BLOCK","DESTROY",
-				    KLISTEND};
-  new DLibFunRetNew(lib::widget_event,string("WIDGET_EVENT"),1,widget_eventKey);
-
-  const string widget_ControlKey[] = {"REALIZE","MANAGED","EVENT_FUNC","EVENT_PRO",
-				      "XMANAGER_ACTIVE_COMMAND","DESTROY",
-				      "GET_UVALUE","SET_UVALUE","SET_VALUE",
-				      "MAP","FUNC_GET_VALUE","PRO_SET_VALUE",
-				      "SET_UNAME","NO_COPY","SET_BUTTON",
-				      "SET_DROPLIST_SELECT","SENSITIVE",
-				      "GET_VALUE","NO_NEWLINE",KLISTEND};
-  const string widget_WarnControlKey[] ={"DEFAULT_FONT","CLEAR_EVENTS","XOFFSET","YOFFSET","TLB_SET_TITLE","KILL_NOTIFY",KLISTEND};
-  new DLibPro(lib::widget_control,string("WIDGET_CONTROL"),1, 
-	      widget_ControlKey,widget_WarnControlKey);
-
 }
